--
--
-- SpeedControl
--
-- # Author:  Heady
-- # date: 17.02.2015
-- # last update: 14.08.2016
--
-- > Not for commercial use  < --
-- > Not allowed for edit - Copyright (C) Heady - www.planet-ls.de < --
--


SpeedControl = {};

local modDir = g_currentModDirectory;
SpeedControl.version = "15.0.2";

function SpeedControl:prerequisitesPresent(specializations)
    return true;
end;

function SpeedControl:load(xmlFile)

	self.setSpeedControlKey = SpeedControl.setSpeedControlKey;
	self.setSpeedControl = SpeedControl.setSpeedControl;
	--self.setMotorSpeed = SpeedControl.setMotorSpeed;
	self.setCruiseControlState = Utils.appendedFunction(self.setCruiseControlState, SpeedControl.setCruiseControlState);

	
	self.speedControlKeys = {};
	self.speedControlKeys[1] = {};
	self.speedControlKeys[1].speed = 10;
	self.speedControlKeys[1].isActive = false;
	self.speedControlKeys[1].isSelected = false;

	self.speedControlKeys[2] = {};
	self.speedControlKeys[2].speed = 20;
	self.speedControlKeys[2].isActive = false;
	self.speedControlKeys[2].isSelected = false;

	self.speedControlKeys[3] = {};
	self.speedControlKeys[3].speed = math.ceil(self.motor:getMaximumForwardSpeed()*3.6);
	self.speedControlKeys[3].isActive = false;
	self.speedControlKeys[3].isSelected = true;

	self.speedControlCurrKey = 3;
	
	self.speedControlChangeWaitTime = 0;
	
	if self.cruiseControl ~= nil then
		self.cruiseControl.isActive = false;
		self.cruiseControl.speed = self.speedControlKeys[3].speed;
	end;
	
end;

function SpeedControl:delete()
end;

function SpeedControl:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)	
	local loadString = getXMLString(xmlFile, key.."#speedControlSpeed");
	if loadString ~= nil then
		local i=1;
		local types = Utils.splitString(" ", loadString);
		for k,v in pairs(types) do
			self.speedControlKeys[i].speed = tonumber(v);
			i = i+1;
		end;
	end;
	self.cruiseControl.speed = self.speedControlKeys[3].speed;
    return BaseMission.VEHICLE_LOAD_OK;
end;
function SpeedControl:getSaveAttributesAndNodes(nodeIdent)	
	local attributes = 'speedControlSpeed="'..math.ceil(self.speedControlKeys[1].speed).." "..math.ceil(self.speedControlKeys[2].speed).." "..math.ceil(self.speedControlKeys[3].speed).. '"';
    return attributes, nil;
end;
	
function SpeedControl:mouseEvent(posX, posY, isDown, isUp, button)
end;

function SpeedControl:keyEvent(unicode, sym, modifier, isDown)
end;

function SpeedControl:readStream(streamId, connection)
    if connection:getIsServer() then
        self.speedControlCurrKey = streamReadInt8(streamId);

		self.speedControlKeys[1].isSelected = streamReadBool(streamId);
		self.speedControlKeys[1].isActive = streamReadBool(streamId);
		self.speedControlKeys[1].speed = streamReadInt8(streamId);
		
		self.speedControlKeys[2].isSelected = streamReadBool(streamId);
		self.speedControlKeys[2].isActive = streamReadBool(streamId);
		self.speedControlKeys[2].speed = streamReadInt8(streamId);
		
		self.speedControlKeys[3].isSelected = streamReadBool(streamId);
		self.speedControlKeys[3].isActive = streamReadBool(streamId);
		self.speedControlKeys[3].speed = streamReadInt8(streamId);
		
		self.cruiseControl.speed = math.ceil(self.speedControlKeys[self.speedControlCurrKey].speed);
    end
end;

function SpeedControl:writeStream(streamId, connection)
    if not connection:getIsServer() then
        streamWriteInt8(streamId, self.speedControlCurrKey);
		
		streamWriteBool(streamId, self.speedControlKeys[1].isSelected);
		streamWriteBool(streamId, self.speedControlKeys[1].isActive);
		streamWriteInt8(streamId, self.speedControlKeys[1].speed);
		
		streamWriteBool(streamId, self.speedControlKeys[2].isSelected);
		streamWriteBool(streamId, self.speedControlKeys[2].isActive);
		streamWriteInt8(streamId, self.speedControlKeys[2].speed);
		
		streamWriteBool(streamId, self.speedControlKeys[3].isSelected);
		streamWriteBool(streamId, self.speedControlKeys[3].isActive);
		streamWriteInt8(streamId, self.speedControlKeys[3].speed);
    end;
end;

function SpeedControl:update(dt)

	if self.isEntered and self.isClient and not self.isHired then
        if self:getIsActiveForInput(false) then
			
			local currSpeed = self.speedControlKeys[self.speedControlCurrKey].speed;
		
			if InputBinding.hasEvent(InputBinding.SPEEDCONTROL_SPEED1) then
				self.speedControlCurrKey = 1;
				self:setSpeedControlKey(1);
				SpeedControlMPEvent:updateSendEvent(self);
			end;
			if InputBinding.hasEvent(InputBinding.SPEEDCONTROL_SPEED2) then
				self.speedControlCurrKey = 2;
				self:setSpeedControlKey(2);
				SpeedControlMPEvent:updateSendEvent(self);
			end;
			if InputBinding.hasEvent(InputBinding.SPEEDCONTROL_SPEED3) then
				self.speedControlCurrKey = 3;
				self:setSpeedControlKey(3);
				SpeedControlMPEvent:updateSendEvent(self);
			end;
			
			
			if InputBinding.hasEvent(InputBinding.SPEEDCONTROL_MINUS) and self.speedControlKeys[self.speedControlCurrKey].speed > 1 then
				self.speedControlKeys[self.speedControlCurrKey].speed = self.speedControlKeys[self.speedControlCurrKey].speed - 1;			
			end;
			
			if InputBinding.hasEvent(InputBinding.SPEEDCONTROL_PLUS) and self.speedControlKeys[self.speedControlCurrKey].speed < math.ceil(self.motor:getMaximumForwardSpeed()*3.6) then
				self.speedControlKeys[self.speedControlCurrKey].speed = self.speedControlKeys[self.speedControlCurrKey].speed + 1;			
			end;
			

			if InputBinding.isPressed(InputBinding.SPEEDCONTROL_MINUS) and self.speedControlKeys[self.speedControlCurrKey].speed > 0 then
				self.speedControlChangeWaitTime = self.speedControlChangeWaitTime + dt; -- realtime ms
				if self.speedControlChangeWaitTime > 250 then
					self.speedControlKeys[self.speedControlCurrKey].speed = self.speedControlKeys[self.speedControlCurrKey].speed - 10*dt/1000
				end;

				if self.speedControlKeys[self.speedControlCurrKey].speed < 1 then
					self.speedControlKeys[self.speedControlCurrKey].speed = 1;
				end;

			elseif InputBinding.isPressed(InputBinding.SPEEDCONTROL_PLUS) and self.speedControlKeys[self.speedControlCurrKey].speed < math.ceil(self.motor:getMaximumForwardSpeed()*3.6) then				
				self.speedControlChangeWaitTime = self.speedControlChangeWaitTime + dt; -- realtime ms
				if self.speedControlChangeWaitTime > 250 then
					self.speedControlKeys[self.speedControlCurrKey].speed = self.speedControlKeys[self.speedControlCurrKey].speed + 10*dt/1000
				end;
				
				if self.speedControlKeys[self.speedControlCurrKey].speed > math.ceil(self.motor:getMaximumForwardSpeed()*3.6) then
					self.speedControlKeys[self.speedControlCurrKey].speed = math.ceil(self.motor:getMaximumForwardSpeed()*3.6);
				end;
			else
				self.speedControlChangeWaitTime = 0;
			end;			
			
			if self.speedControlKeys[self.speedControlCurrKey].speed ~= currSpeed then
				--print("self.speedControlKeys[self.speedControlCurrKey].speed: ", self.speedControlKeys[self.speedControlCurrKey].speed)				
				self.cruiseControl.speed = math.ceil(self.speedControlKeys[self.speedControlCurrKey].speed);
				--print("self.cruiseControl.speed: ", self.cruiseControl.speed)
				SpeedControlSpeedMPEvent:updateSendEvent(self);				
			end;
			
			--self:setMotorSpeed(self.speedControlKeys[self.speedControlCurrKey].speed);

			
		end;
	end;

end;

function SpeedControl:updateTick(dt)

end;

function SpeedControl:draw()

	if self.isEntered then

		if self.cruiseControl.state == 1 then
			setTextColor(0.2, 0.83, 0, 1);
		else
			setTextColor(1,1,1,1);
		end;
		setTextBold(true);
		setTextAlignment(RenderText.ALIGN_LEFT)
		renderText(g_currentMission.vehicleHudPosX  + 236/1920, g_currentMission.vehicleHudPosY + 29/1200, 13/1200, tostring(self.speedControlCurrKey));
		
		setTextColor(1,1,1,1);
		setTextBold(false);
		--setTextAlignment(RenderText.ALIGN_LEFT)

	end;	

end;

function SpeedControl:setSpeedControlKey(key)

	if self.speedControlKeys[key].isActive then
		self.speedControlKeys[key].isActive = false;
		
		self:setSpeedControl(key, false);
	else
		if self.speedControlKeys[key].isSelected then
			self.speedControlKeys[key].isActive = true;
			
			self:setSpeedControl(key, true);
		else
			for i=1, table.getn(self.speedControlKeys) do
				self.speedControlKeys[i].isSelected = false;	
			end;
			for i=1, table.getn(self.speedControlKeys) do
				self.speedControlKeys[i].isActive = false;	
			end;
			self.speedControlKeys[key].isSelected = true;	
			
			self:setSpeedControl(key, false);
		end;
	end;

end;

function SpeedControl:setSpeedControl(key, active)

	if active then
		self.cruiseControl.state = 1;
		self.motor:setSpeedLimit(self.cruiseControl.speed);
	else
		self.cruiseControl.state = 0;
		self.motor:setSpeedLimit(math.huge);
	end;

end;

function SpeedControl:setCruiseControlState(state, noEventSend)

	for i=1, table.getn(self.speedControlKeys) do
		self.speedControlKeys[i].isActive = false;	
	end;
	
end;


SpeedControlMPEvent = {};
SpeedControlMPEvent_mt = Class(SpeedControlMPEvent, Event);

InitEventClass(SpeedControlMPEvent, "SpeedControlMPEvent");

function SpeedControlMPEvent:emptyNew()
    local self = Event:new(SpeedControlMPEvent_mt);
    self.className="SpeedControlMPEvent";
    return self;
end;

function SpeedControlMPEvent:new(object)
    local self = SpeedControlMPEvent:emptyNew()
    self.object = object;
    return self;
end;

function SpeedControlMPEvent:readStream(streamId, connection)
    self.object = networkGetObject(streamReadInt32(streamId));	
	
	self.object.speedControlCurrKey = streamReadInt8(streamId);
	
	self.object:setSpeedControlKey(self.object.speedControlCurrKey);
	
	if not connection:getIsServer() then -- if server read then send to clients
		g_server:broadcastEvent(SpeedControlMPEvent:new(self.object), nil, connection, self.object);
	end;

end;

function SpeedControlMPEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.object));	
	
	streamWriteInt8(streamId, self.object.speedControlCurrKey);	
end;

function SpeedControlMPEvent:updateSendEvent(runSelf)	

	if g_server ~= nil then
		g_server:broadcastEvent(SpeedControlMPEvent:new(runSelf));
	else
		g_client:getServerConnection():sendEvent(SpeedControlMPEvent:new(runSelf));
	end;

end;

SpeedControlSpeedMPEvent = {};
SpeedControlSpeedMPEvent_mt = Class(SpeedControlSpeedMPEvent, Event);

InitEventClass(SpeedControlSpeedMPEvent, "SpeedControlSpeedMPEvent");

function SpeedControlSpeedMPEvent:emptyNew()
    local self = Event:new(SpeedControlSpeedMPEvent_mt);
    self.className="SpeedControlSpeedMPEvent";
    return self;
end;

function SpeedControlSpeedMPEvent:new(object)
    local self = SpeedControlSpeedMPEvent:emptyNew()
    self.object = object;
    return self;
end;

function SpeedControlSpeedMPEvent:readStream(streamId, connection)
    self.object = networkGetObject(streamReadInt32(streamId));	
	
	self.object.speedControlKeys[self.object.speedControlCurrKey].speed = streamReadInt8(streamId);
	
	self.object.cruiseControl.speed = math.ceil(self.object.speedControlKeys[self.object.speedControlCurrKey].speed);
	
	if not connection:getIsServer() then -- if server read then send to clients
		g_server:broadcastEvent(SpeedControlSpeedMPEvent:new(self.object), nil, connection, self.object);
	end;

end;

function SpeedControlSpeedMPEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.object));	
	
	streamWriteInt8(streamId, self.object.speedControlKeys[self.object.speedControlCurrKey].speed);
end;

function SpeedControlSpeedMPEvent:updateSendEvent(runSelf)	

	if g_server ~= nil then
		g_server:broadcastEvent(SpeedControlSpeedMPEvent:new(runSelf));
	else
		g_client:getServerConnection():sendEvent(SpeedControlSpeedMPEvent:new(runSelf));
	end;

end;


